@@ -1,4 +1,4 @@
-/* $Id: cookie.c,v 1.7 2002/01/10 15:39:21 ukai Exp $ */
+/* $Id: cookie.c,v 1.9 2003/09/26 17:59:51 ukai Exp $ */
 
 /*
  * References for version 0 cookie:                                  
@@ -417,7 +417,7 @@ save_cookies(void)
 
     check_expired_cookies();
 
-    if (!First_cookie || is_saved || rc_dir_is_tmp)
+    if (!First_cookie || is_saved || no_rc_dir)
 	return;
 
     cookie_file = rcFile(COOKIE_FILE);
@@ -537,6 +537,7 @@ initCookie(void)
 Buffer *
 cookie_list_panel(void)
 {
+    /* FIXME: gettextize? */
     Str src = Strnew_charp("<html><head><title>Cookies</title></head>"
 			   "<body><center><b>Cookies</b></center>"
 			   "<p><form method=internal action=cookie>");
