@@ -1,4 +1,4 @@
-/* $Id: rc.c,v 1.94 2003/09/26 20:48:22 ukai Exp $ */
+/* $Id: rc.c,v 1.96 2004/04/22 16:34:08 ukai Exp $ */
 /* 
  * Initialization file etc.
  */
@@ -246,19 +246,6 @@ struct sel_c {
 
 #ifdef USE_COLOR
 static struct sel_c colorstr[] = {
-#if LANG == JA
-    {0, "black", ""},
-    {1, "red", ""},
-    {2, "green", ""},
-    {3, "yellow", ""},
-    {4, "blue", ""},
-    {5, "magenta", ""},
-    {6, "cyan", ""},
-    {7, "white", ""},
-    {8, "terminal", "ü"},
-    {0, NULL, NULL}
-#else				/* LANG != JA */
-    /* FIXME: gettextize here */
     {0, "black", N_("black")},
     {1, "red", N_("red")},
     {2, "green", N_("green")},
@@ -269,7 +256,6 @@ static struct sel_c colorstr[] = {
     {7, "white", N_("white")},
     {8, "terminal", N_("terminal")},
     {0, NULL, NULL}
-#endif				/* LANG != JA */
 };
 #endif				/* USE_COLOR */
 
@@ -287,31 +273,17 @@ static char n_s[][2] = {
 
 
 static struct sel_c defaulturls[] = {
-#if LANG == JA
-    {N_S(DEFAULT_URL_EMPTY), "̵"},
-    {N_S(DEFAULT_URL_CURRENT), "ߤURL"},
-    {N_S(DEFAULT_URL_LINK), "URL"},
-#else
-    /* FIXME: gettextize here */
     {N_S(DEFAULT_URL_EMPTY), N_("none")},
     {N_S(DEFAULT_URL_CURRENT), N_("current URL")},
     {N_S(DEFAULT_URL_LINK), N_("link URL")},
-#endif
     {0, NULL, NULL}
 };
 
 #ifdef USE_MOUSE
 static struct sel_c wheelmode[] = {
-#if LANG == JA
-    {TRUE, "1", "A:̥"},
-    {FALSE, "0", "B:ιԿ"},
-    {0, NULL, NULL}
-#else				/* LANG != JA */
-    /* FIXME: gettextize here */
     {TRUE, "1", N_("A:relative to screen height")},
     {FALSE, "0", N_("B:fixed speed")},
     {0, NULL, NULL}
-#endif				/* LANG != JA */
 };
 #endif				/* MOUSE */
 
@@ -691,29 +663,6 @@ struct param_ptr params10[] = {
 #endif
 
 struct param_section sections[] = {
-#if LANG == JA
-    {"ɽط", params1},
-#ifdef USE_COLOR
-    {"ɽ", params2},
-#endif				/* USE_COLOR */
-    {"¿", params3},
-    {"ǥ쥯ȥ", params5},
-    {"ץ", params6},
-    {"ͥåȥ", params9},
-    {"ץ", params4},
-#ifdef USE_SSL
-    {"SSL", params7},
-#endif
-#ifdef USE_COOKIE
-    {"å", params8},
-#endif
-#ifdef USE_M17N
-    {"ʸɤ", params10},
-#endif
-
-#else				/* LANG != JA */
-    /* FIXME: gettextize here */
-
     {N_("Display Settings"), params1},
 #ifdef USE_COLOR
     {N_("Color Settings"), params2},
@@ -732,7 +681,6 @@ struct param_section sections[] = {
 #ifdef USE_M17N
     {N_("Charset Settings"), params10},
 #endif
-#endif				/* LANG != JA */
     {NULL, NULL}
 };
 
@@ -1194,17 +1142,12 @@ sync_with_option(void)
     loadPreForm();
 
     if (AcceptLang == NULL || *AcceptLang == '\0') {
-#if LANG == JA
-	/* FIXME: gettextize here */
-	AcceptLang = "ja;q=1.0, en;q=0.5";
-#else				/* LANG != JA (must be EN) */
 	/* TRANSLATORS: 
 	 * AcceptLang default: this is used in Accept-Language: HTTP request 
 	 * header. For example, ja.po should translate it as
 	 * "ja;q=1.0, en;q=0.5" like that.
 	 */
 	AcceptLang = _("en;q=1.0");
-#endif
     }
     if (AcceptEncoding == NULL || *AcceptEncoding == '\0')
 	AcceptEncoding = acceptableEncoding();
@@ -1377,9 +1320,11 @@ load_option_panel(void)
 		    wc_conv(gettext(p->comment), OptionCharset,
 			    InnerCharset)->ptr;
 	}
+#ifdef USE_COLOR
 	for (s = colorstr; s->text; s++)
 	    s->text = wc_conv(gettext(s->text), OptionCharset,
 			      InnerCharset)->ptr;
+#endif
 	OptionEncode = TRUE;
     }
 #endif
