@@ -1,4 +1,4 @@
-/* $Id: url.c,v 1.87 2003/12/08 16:17:21 ukai Exp $ */
+/* $Id: url.c,v 1.89 2004/04/16 18:47:19 ukai Exp $ */
 #include "fm.h"
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -354,7 +354,8 @@ openSSLHandle(int sock, char *hostname, char **p_cert)
 		goto eend;
 	    }
 	}
-	if (SSL_CTX_load_verify_locations(ssl_ctx, ssl_ca_file, ssl_ca_path))
+	if ((!ssl_ca_file && !ssl_ca_path)
+	    || SSL_CTX_load_verify_locations(ssl_ctx, ssl_ca_file, ssl_ca_path))
 #endif				/* defined(USE_SSL_VERIFY) */
 	    SSL_CTX_set_default_verify_paths(ssl_ctx);
 #endif				/* SSLEAY_VERSION_NUMBER >= 0x0800 */
@@ -1399,33 +1400,6 @@ HTTPrequest(ParsedURL *pu, ParsedURL *current, HRequest *hr, TextList *extra)
 	    Strcat_charp(tmp, i->ptr);
 	}
 
-    if (!seen_www_auth
-#ifdef USE_SSL
-	&& hr->command != HR_COMMAND_CONNECT
-#endif
-	) {
-	Str auth_cookie = find_auth_cookie(pu->host, pu->port, pu->file, NULL);
-	if (auth_cookie)
-	    Strcat_m_charp(tmp, "Authorization: ", auth_cookie->ptr,
-			   "\r\n", NULL);
-    }
-
-    if (!seen_proxy_auth && (hr->flag & HR_FLAG_PROXY)
-#ifdef USE_SSL
-	&& (pu->scheme != SCM_HTTPS || hr->command == HR_COMMAND_CONNECT)
-#endif
-	) {
-	ParsedURL *proxy_pu = schemeToProxy(pu->scheme);
-	Str auth_cookie =
-	    find_auth_cookie(proxy_pu->host, proxy_pu->port, proxy_pu->file,
-			     NULL);
-	if (!auth_cookie && proxy_auth_cookie)
-	    auth_cookie = proxy_auth_cookie;
-	if (auth_cookie)
-	    Strcat_m_charp(tmp, "Proxy-Authorization: ", auth_cookie->ptr,
-			   "\r\n", NULL);
-    }
-
 #ifdef USE_COOKIE
     if (hr->command != HR_COMMAND_CONNECT &&
 	use_cookie && (cookie = find_cookie(pu))) {
