@@ -1,4 +1,4 @@
-/* $Id: w3mbookmark.c,v 1.8 2003/01/15 17:13:22 ukai Exp $ */
+/* $Id: w3mbookmark.c,v 1.10 2003/09/23 18:42:25 ukai Exp $ */
 #include <stdlib.h>
 #include <stdio.h>
 #include "config.h"
@@ -8,6 +8,7 @@
 #include "parsetag.h"
 
 #if LANG == JA
+/* FIXME: gettextize here */
 #define BKMARK_TITLE "֥åޡϿ"
 #define BKMARK_ADD "Ͽ"
 #define DEFAULT_SECTION "̤ʬ"
@@ -57,7 +58,12 @@ print_bookmark_panel(char *bmark, char *url, char *title)
     FILE *f;
     char *p;
 
+#if LANG == JA
+    /* FIXME: gettextize here */
+    printf("Content-Type: text/html; charset=EUC-JP\n\n");
+#else
     printf("Content-Type: text/html\n\n");
+#endif
     printf(bkmark_src1, html_quote(bmark), html_quote(Local_cookie));
     if ((f = fopen(bmark, "r")) != NULL) {
 	printf("<tr><td>Section:<td><select name=\"section\">\n");
