@@ -1,4 +1,4 @@
-/* $Id: fb_w3mimg.c,v 1.8 2002/12/25 16:18:19 ukai Exp $ */
+/* $Id: fb_w3mimg.c,v 1.13 2003/08/29 15:06:52 ukai Exp $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -60,6 +60,15 @@ w3mfb_close(w3mimg_op * self)
     fb_close();
 }
 
+static int
+w3mfb_clear(w3mimg_op * self, int x, int y, int w, int h)
+{
+    if (self == NULL)
+	return 0;
+    fb_image_clear(x, y, w, h);
+    return 1;
+}
+
 static int
 w3mfb_load_image(w3mimg_op * self, W3MImage * img, char *fname, int w, int h)
 {
@@ -67,7 +76,7 @@ w3mfb_load_image(w3mimg_op * self, W3MImage * img, char *fname, int w, int h)
 
     if (self == NULL)
 	return 0;
-    im = fb_image_load(fname, w, h);
+    im = fb_image_load(fname, w, h, self->max_anim);
     if (!im)
 	return 0;
     img->pixmap = im;
@@ -162,7 +171,7 @@ w3mimg_fbopen()
 	return NULL;
     memset(wop, 0, sizeof(w3mimg_op));
 
-    if (!check_tty_console(getenv("W3M_TTY"))) {
+    if (!check_tty_console(getenv("W3M_TTY")) && strcmp("jfbterm", getenv("TERM")) != 0) {
 	fprintf(stderr, "w3mimgdisplay/fb: tty is not console\n");
 	goto error;
     }
@@ -179,6 +188,7 @@ w3mimg_fbopen()
     wop->set_background = w3mfb_set_background;
     wop->sync = w3mfb_sync;
     wop->close = w3mfb_close;
+    wop->clear = w3mfb_clear;
 
     wop->load_image = w3mfb_load_image;
     wop->show_image = w3mfb_show_image;
