@@ -1,4 +1,4 @@
-/* $Id: x11_w3mimg.c,v 1.12 2003/01/21 15:38:59 ukai Exp $ */
+/* $Id: x11_w3mimg.c,v 1.25 2003/07/13 16:19:10 ukai Exp $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -38,8 +38,43 @@ struct x11_image {
     int total;
     int no;
     int wait;
+    int delay;
     Pixmap *pixmap;
 };
+
+static void
+get_animation_size(GdkPixbufAnimation * animation, int *w, int *h, int *delay)
+{
+    GList *frames;
+    int iw, ih, n, i, d = -1;
+
+    frames = gdk_pixbuf_animation_get_frames(animation);
+    n = gdk_pixbuf_animation_get_num_frames(animation);
+    *w = gdk_pixbuf_animation_get_width(animation);
+    *h = gdk_pixbuf_animation_get_height(animation);
+    for (i = 0; i < n; i++) {
+	GdkPixbufFrame *frame;
+	GdkPixbuf *pixbuf;
+	int tmp;
+
+	frame = (GdkPixbufFrame *) g_list_nth_data(frames, i);
+	tmp = gdk_pixbuf_frame_get_delay_time(frame);
+	if (tmp > d)
+	    d = tmp;
+	pixbuf = gdk_pixbuf_frame_get_pixbuf(frame);
+	iw = gdk_pixbuf_frame_get_x_offset(frame)
+	    + gdk_pixbuf_get_width(pixbuf);
+	ih = gdk_pixbuf_frame_get_y_offset(frame)
+	    + gdk_pixbuf_get_height(pixbuf);
+	if (iw > *w)
+	    *w = iw;
+	if (ih > *h)
+	    *h = ih;
+    }
+    if (delay)
+	*delay = d;
+}
+
 #endif
 
 static int
@@ -87,6 +122,25 @@ x11_finish(w3mimg_op * self)
     return 1;
 }
 
+static int
+x11_clear(w3mimg_op * self, int x, int y, int w, int h)
+{
+    struct x11_info *xi;
+    if (self == NULL)
+	return 0;
+    xi = (struct x11_info *)self->priv;
+    if (xi == NULL)
+	return 0;
+
+    if (x < 0)
+	x = 0;
+    if (y < 0)
+	y = 0;
+
+    XClearArea(xi->display, xi->window, x, y, w, h, False);
+    return 1;
+}
+
 static int
 x11_active(w3mimg_op * self)
 {
@@ -187,6 +241,7 @@ x11_img_new(struct x11_info *xi, int w, int h, int n)
     img->no = 0;
     img->total = n;
     img->wait = 0;
+    img->delay = -1;
 
     return img;
   ERROR:
@@ -236,10 +291,10 @@ x11_load_image(w3mimg_op * self, W3MImage * img, char *fname, int w, int h)
 #elif defined(USE_GDKPIXBUF)
     GdkPixbufAnimation *animation;
     GList *frames;
-    int i, iw, ih, n;
+    int i, j, iw, ih, n, frame_num, delay, max_anim;
     double ratio_w, ratio_h;
     struct x11_image *ximg;
-    GdkPixbufFrameAction action = GDK_PIXBUF_FRAME_REVERT;
+    Pixmap tmp_pixmap;
 #endif
 
     if (self == NULL)
@@ -267,32 +322,42 @@ x11_load_image(w3mimg_op * self, W3MImage * img, char *fname, int w, int h)
 #elif defined(USE_IMLIB2)
     im = imlib_load_image(fname);
     if (!im)
-        return 0;
+	return 0;
     imlib_context_set_image(im);
     if (w <= 0)
-        w = imlib_image_get_width();
+	w = imlib_image_get_width();
     if (h <= 0)
-        h = imlib_image_get_height();
+	h = imlib_image_get_height();
     img->pixmap = (void *)XCreatePixmap(xi->display, xi->parent, w, h,
 					DefaultDepth(xi->display, 0));
     if (!img->pixmap)
-        return 0;
+	return 0;
     XSetForeground(xi->display, xi->imageGC, xi->background_pixel);
     XFillRectangle(xi->display, (Pixmap) img->pixmap, xi->imageGC, 0, 0, w, h);
     imlib_context_set_display(xi->display);
     imlib_context_set_visual(DefaultVisual(xi->display, 0));
     imlib_context_set_colormap(DefaultColormap(xi->display, 0));
-    imlib_context_set_drawable((Drawable)img->pixmap);
+    imlib_context_set_drawable((Drawable) img->pixmap);
     imlib_render_image_on_drawable_at_size(0, 0, w, h);
     imlib_free_image();
 #elif defined(USE_GDKPIXBUF)
+    max_anim = self->max_anim;
     animation = gdk_pixbuf_animation_new_from_file(fname);
     if (!animation)
 	return 0;
     frames = gdk_pixbuf_animation_get_frames(animation);
-    n = gdk_pixbuf_animation_get_num_frames(animation);
-    iw = gdk_pixbuf_animation_get_width(animation);
-    ih = gdk_pixbuf_animation_get_height(animation);
+    frame_num = n = gdk_pixbuf_animation_get_num_frames(animation);
+
+    get_animation_size(animation, &iw, &ih, &delay);
+    if (delay <= 0)
+	max_anim = -1;
+
+    if (max_anim < 0) {
+	frame_num = (-max_anim > n) ? n : -max_anim;
+    }
+    else if (max_anim > 0) {
+	frame_num = n = (max_anim > n) ? n : max_anim;
+    }
 
     if (w < 1 || h < 1) {
 	w = iw;
@@ -303,20 +368,35 @@ x11_load_image(w3mimg_op * self, W3MImage * img, char *fname, int w, int h)
 	ratio_w = 1.0 * w / iw;
 	ratio_h = 1.0 * h / ih;
     }
-    ximg = x11_img_new(xi, w, h, n);
-    if (!ximg) {
+    tmp_pixmap = XCreatePixmap(xi->display, xi->parent, w, h,
+			       DefaultDepth(xi->display, 0));
+    XFillRectangle(xi->display, (Pixmap) tmp_pixmap, xi->imageGC, 0, 0, w, h);
+    if (!tmp_pixmap) {
 	gdk_pixbuf_animation_unref(animation);
 	return 0;
     }
-    for (i = 0; i < n; i++) {
+    ximg = x11_img_new(xi, w, h, frame_num);
+    if (!ximg) {
+	XFreePixmap(xi->display, tmp_pixmap);
+	gdk_pixbuf_animation_unref(animation);
+	return 0;
+    }
+    for (j = 0; j < n; j++) {
 	GdkPixbufFrame *frame;
 	GdkPixbuf *org_pixbuf, *pixbuf;
 	int width, height, ofstx, ofsty;
 
-	frame = (GdkPixbufFrame *) g_list_nth_data(frames, i);
+	if (max_anim < 0) {
+	    i = (j - n + frame_num > 0) ? (j - n + frame_num) : 0;
+	}
+	else {
+	    i = j;
+	}
+	frame = (GdkPixbufFrame *) g_list_nth_data(frames, j);
 	org_pixbuf = gdk_pixbuf_frame_get_pixbuf(frame);
 	ofstx = gdk_pixbuf_frame_get_x_offset(frame);
 	ofsty = gdk_pixbuf_frame_get_y_offset(frame);
+	delay = gdk_pixbuf_frame_get_delay_time(frame);
 	width = gdk_pixbuf_get_width(org_pixbuf);
 	height = gdk_pixbuf_get_height(org_pixbuf);
 
@@ -332,36 +412,41 @@ x11_load_image(w3mimg_op * self, W3MImage * img, char *fname, int w, int h)
 	width = gdk_pixbuf_get_width(pixbuf);
 	height = gdk_pixbuf_get_height(pixbuf);
 
-	if (i > 0) {
-	    switch (action) {
-	    case GDK_PIXBUF_FRAME_RETAIN:
-		XCopyArea(xi->display, ximg->pixmap[i - 1], ximg->pixmap[i],
-			  xi->imageGC, 0, 0, w, h, 0, 0);
-		break;
-	    case GDK_PIXBUF_FRAME_DISPOSE:
-		break;
-	    case GDK_PIXBUF_FRAME_REVERT:
-		XCopyArea(xi->display, ximg->pixmap[0], ximg->pixmap[i],
-			  xi->imageGC, 0, 0, w, h, 0, 0);
-		break;
-	    default:
-		XCopyArea(xi->display, ximg->pixmap[0], ximg->pixmap[i],
-			  xi->imageGC, 0, 0, w, h, 0, 0);
-		break;
-	    }
-	}
+	if (delay > ximg->delay)
+	    ximg->delay = delay;
 
+	XCopyArea(xi->display, tmp_pixmap, ximg->pixmap[i],
+		  xi->imageGC, 0, 0, w, h, 0, 0);
 	gdk_pixbuf_xlib_render_to_drawable_alpha(pixbuf,
 						 (Drawable) ximg->pixmap[i], 0,
 						 0, ofstx, ofsty, width,
 						 height,
 						 GDK_PIXBUF_ALPHA_BILEVEL, 1,
 						 XLIB_RGB_DITHER_NORMAL, 0, 0);
-	action = gdk_pixbuf_frame_get_action(frame);
+
+	switch (gdk_pixbuf_frame_get_action(frame)) {
+	case GDK_PIXBUF_FRAME_RETAIN:
+	    XCopyArea(xi->display, ximg->pixmap[i], tmp_pixmap,
+		      xi->imageGC, 0, 0, w, h, 0, 0);
+	    break;
+	case GDK_PIXBUF_FRAME_DISPOSE:
+	    break;
+	case GDK_PIXBUF_FRAME_REVERT:
+	    XCopyArea(xi->display, ximg->pixmap[0], tmp_pixmap,
+		      xi->imageGC, 0, 0, w, h, 0, 0);
+	    break;
+	default:
+	    XCopyArea(xi->display, ximg->pixmap[0], tmp_pixmap,
+		      xi->imageGC, 0, 0, w, h, 0, 0);
+	    break;
+	}
+
+
 	if (org_pixbuf != pixbuf)
 	    gdk_pixbuf_finalize(pixbuf);
 
     }
+    XFreePixmap(xi->display, tmp_pixmap);
     gdk_pixbuf_animation_unref(animation);
     img->pixmap = ximg;
 #endif
@@ -397,7 +482,10 @@ x11_show_image(w3mimg_op * self, W3MImage * img, int sx, int sy, int sw,
 	      (sh ? sh : img->height), x + self->offset_x, y + self->offset_y);
 #elif defined(USE_GDKPIXBUF)
 #define WAIT_CNT 4
-    i = ximg->no;
+    if (ximg->delay <= 0)
+	i = ximg->total - 1;
+    else
+	i = ximg->no;
     XCopyArea(xi->display, ximg->pixmap[i], xi->window, xi->imageGC,
 	      sx, sy,
 	      (sw ? sw : img->width),
@@ -462,7 +550,7 @@ x11_get_image_size(w3mimg_op * self, W3MImage * img, char *fname, int *w,
 #elif defined(USE_IMLIB2)
     Imlib_Image im;
 #elif defined(USE_GDKPIXBUF)
-    GdkPixbuf *pixbuf;
+    GdkPixbufAnimation *animation;
 #endif
 
     if (self == NULL)
@@ -482,21 +570,19 @@ x11_get_image_size(w3mimg_op * self, W3MImage * img, char *fname, int *w,
 #elif defined(USE_IMLIB2)
     im = imlib_load_image(fname);
     if (im == NULL)
-        return 0;
+	return 0;
 
     imlib_context_set_image(im);
     *w = imlib_image_get_width();
     *h = imlib_image_get_height();
     imlib_free_image();
 #elif defined(USE_GDKPIXBUF)
-    pixbuf = gdk_pixbuf_new_from_file(fname);
-    if (!pixbuf)
+    animation = gdk_pixbuf_animation_new_from_file(fname);
+    if (!animation)
 	return 0;
 
-    *w = gdk_pixbuf_get_width(pixbuf);
-    *h = gdk_pixbuf_get_height(pixbuf);
-
-    gdk_pixbuf_unref(pixbuf);
+    get_animation_size(animation, w, h, NULL);
+    gdk_pixbuf_animation_unref(animation);
 #endif
     return 1;
 }
@@ -621,6 +707,7 @@ w3mimg_x11open()
     wop->set_background = x11_set_background;
     wop->sync = x11_sync;
     wop->close = x11_close;
+    wop->clear = x11_clear;
 
     wop->load_image = x11_load_image;
     wop->show_image = x11_show_image;
